@@ -232,13 +232,8 @@ class Source(object):
             for node in ast.walk(self.tree):
                 for child in ast.iter_child_nodes(node):
                     child.parent = node
-                if hasattr(node, "lineno"):
-                    if hasattr(node, "end_lineno") and isinstance(node, ast.expr):
-                        linenos = range(node.lineno, node.end_lineno + 1)
-                    else:
-                        linenos = [node.lineno]
-                    for lineno in linenos:
-                        self._nodes_by_line[lineno].append(node)
+                for lineno in node_linenos(node):
+                    self._nodes_by_line[lineno].append(node)
 
             visitor = QualnameVisitor()
             visitor.visit(self.tree)
@@ -323,10 +318,7 @@ class Source(object):
                     try:
                         stmts = source.statements_at_line(lineno)
                         if stmts:
-                            if code.co_name == "<module>" and re.search(
-                                r"<ipython-input-|[/\\]ipykernel_\d+[/\\]",
-                                code.co_filename,
-                            ):
+                            if is_ipython_cell_code(code):
                                 for stmt in stmts:
                                     tree = _extract_ipython_statement(stmt)
                                     try:
@@ -579,11 +571,13 @@ class NodeFinder(object):
         self.instruction = instruction = self.get_actual_current_instruction(lasti)
         op_name = instruction.opname
         extra_filter = lambda e: True
+        ctx = type(None)
 
         if op_name.startswith('CALL_'):
             typ = ast.Call
         elif op_name.startswith(('BINARY_SUBSCR', 'SLICE+')):
             typ = ast.Subscript
+            ctx = ast.Load
         elif op_name.startswith('BINARY_'):
             typ = ast.BinOp
             op_type = dict(
@@ -613,15 +607,23 @@ class NodeFinder(object):
             extra_filter = lambda e: isinstance(e.op, op_type)
         elif op_name in ('LOAD_ATTR', 'LOAD_METHOD', 'LOOKUP_METHOD'):
             typ = ast.Attribute
-            # `in` to handle private mangled attributes
-            extra_filter = lambda e: e.attr in instruction.argval
+            ctx = ast.Load
+            extra_filter = lambda e: attr_names_match(e.attr, instruction.argval)
         elif op_name in ('LOAD_NAME', 'LOAD_GLOBAL', 'LOAD_FAST', 'LOAD_DEREF', 'LOAD_CLASSDEREF'):
             typ = ast.Name
+            ctx = ast.Load
             if PY3 or instruction.argval:
                 extra_filter = lambda e: e.id == instruction.argval
         elif op_name in ('COMPARE_OP', 'IS_OP', 'CONTAINS_OP'):
             typ = ast.Compare
             extra_filter = lambda e: len(e.ops) == 1
+        elif op_name.startswith(('STORE_SLICE', 'STORE_SUBSCR')):
+            ctx = ast.Store
+            typ = ast.Subscript
+        elif op_name.startswith('STORE_ATTR'):
+            ctx = ast.Store
+            typ = ast.Attribute
+            extra_filter = lambda e: attr_names_match(e.attr, instruction.argval)
         else:
             raise RuntimeError(op_name)
 
@@ -631,11 +633,18 @@ class NodeFinder(object):
                 for stmt in stmts
                 for node in ast.walk(stmt)
                 if isinstance(node, typ)
-                if not (hasattr(node, "ctx") and not isinstance(node.ctx, ast.Load))
+                if isinstance(getattr(node, "ctx", None), ctx)
                 if extra_filter(node)
                 if statement_containing_node(node) == stmt
             }
 
+            if ctx == ast.Store:
+                # No special bytecode tricks here.
+                # We can handle multiple assigned attributes with different names,
+                # but only one assigned subscript.
+                self.result = only(exprs)
+                return
+
             matching = list(self.matching_nodes(exprs))
             if not matching and typ == ast.Call:
                 self.find_decorator(stmts)
@@ -789,9 +798,9 @@ class NodeFinder(object):
     def find_codes(self, root_code):
         checks = [
             attrgetter('co_firstlineno'),
-            attrgetter('co_name'),
             attrgetter('co_freevars'),
             attrgetter('co_cellvars'),
+            lambda c: is_ipython_cell_code_name(c.co_name) or c.co_name,
         ]
         if not self.is_pytest:
             checks += [
@@ -1055,10 +1064,10 @@ def assert_linenos(tree):
     for node in ast.walk(tree):
         if (
                 hasattr(node, 'parent') and
-                hasattr(node, 'lineno') and
                 isinstance(statement_containing_node(node), ast.Assert)
         ):
-            yield node.lineno
+            for lineno in node_linenos(node):
+                yield lineno
 
 
 def _extract_ipython_statement(stmt):
@@ -1074,3 +1083,41 @@ def _extract_ipython_statement(stmt):
     tree.body = [stmt]
     ast.copy_location(tree, stmt)
     return tree
+
+
+def is_ipython_cell_code_name(code_name):
+    return bool(re.match(r"(<module>|<cell line: \d+>)$", code_name))
+
+
+def is_ipython_cell_filename(filename):
+    return re.search(r"<ipython-input-|[/\\]ipykernel_\d+[/\\]", filename)
+
+
+def is_ipython_cell_code(code_obj):
+    return (
+        is_ipython_cell_filename(code_obj.co_filename) and
+        is_ipython_cell_code_name(code_obj.co_name)
+    )
+
+
+def attr_names_match(attr, argval):
+    """
+    Checks that the user-visible attr (from ast) can correspond to
+    the argval in the bytecode, i.e. the real attribute fetched internally,
+    which may be mangled for private attributes.
+    """
+    if attr == argval:
+        return True
+    if not attr.startswith("__"):
+        return False
+    return bool(re.match(r"^_\w+%s$" % attr, argval))
+
+
+def node_linenos(node):
+    if hasattr(node, "lineno"):
+        if hasattr(node, "end_lineno") and isinstance(node, ast.expr):
+            linenos = range(node.lineno, node.end_lineno + 1)
+        else:
+            linenos = [node.lineno]
+        for lineno in linenos:
+            yield lineno
